@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,7 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#[cfg(any(test, feature = "family-command"))]
+#[cfg(feature = "family-command")]
 pub mod command;
+#[cfg(feature = "family-smallbank")]
+pub mod smallbank;
 #[cfg(feature = "family-xo")]
 pub mod xo;
