@@ -1,16 +1,15 @@
+use crate::error::InvalidStateError;
 use crate::protocol::batch::BatchBuilder;
 use crate::protocol::batch::BatchPair;
 use crate::protocol::transaction::HashMethod;
 use crate::protocol::transaction::TransactionBuilder;
 use crate::protocol::transaction::TransactionPair;
-use crate::workload::error::WorkloadError;
 use crate::workload::BatchWorkload;
+use crate::workload::ExpectedBatchResult;
 use crate::workload::TransactionWorkload;
 
 use cylinder::{secp256k1::Secp256k1Context, Context, Signer};
-use rand::distributions::Alphanumeric;
-use rand::prelude::*;
-use rand_hc::Hc128Rng;
+use rand::{distributions::Alphanumeric, prelude::*};
 use sha2::{Digest, Sha512};
 
 static FAMILY_NAME: &str = "xo";
@@ -18,7 +17,7 @@ static FAMILY_VERSION: &str = "0.1";
 static NONCE_SIZE: usize = 32;
 
 pub struct XoTransactionWorkload {
-    rng: Hc128Rng,
+    rng: StdRng,
     signer: Box<dyn Signer>,
 }
 
@@ -31,8 +30,8 @@ impl Default for XoTransactionWorkload {
 impl XoTransactionWorkload {
     pub fn new(seed: Option<u64>, signer: Option<Box<dyn Signer>>) -> Self {
         let rng = match seed {
-            Some(seed) => Hc128Rng::seed_from_u64(seed),
-            None => Hc128Rng::from_entropy(),
+            Some(seed) => StdRng::seed_from_u64(seed),
+            None => StdRng::from_entropy(),
         };
         let signer = signer.unwrap_or_else(new_signer);
 
@@ -41,25 +40,36 @@ impl XoTransactionWorkload {
 }
 
 impl TransactionWorkload for XoTransactionWorkload {
-    fn next_transaction(&mut self) -> Result<TransactionPair, WorkloadError> {
-        let nonce = self
-            .rng
-            .sample_iter(&Alphanumeric)
+    fn next_transaction(
+        &mut self,
+    ) -> Result<(TransactionPair, Option<ExpectedBatchResult>), InvalidStateError> {
+        let nonce = std::iter::repeat(())
+            .map(|()| self.rng.sample(Alphanumeric))
+            .map(char::from)
             .take(NONCE_SIZE)
             .collect::<String>()
             .into_bytes();
 
         let payload = Payload::new_as_create_with_random_name(&mut self.rng);
 
-        Ok(TransactionBuilder::new()
-            .with_family_name(FAMILY_NAME.to_string())
-            .with_family_version(FAMILY_VERSION.to_string())
-            .with_inputs(payload.inputs())
-            .with_outputs(payload.outputs())
-            .with_nonce(nonce)
-            .with_payload(payload.bytes())
-            .with_payload_hash_method(HashMethod::SHA512)
-            .build_pair(&*self.signer)?)
+        Ok((
+            TransactionBuilder::new()
+                .with_family_name(FAMILY_NAME.to_string())
+                .with_family_version(FAMILY_VERSION.to_string())
+                .with_inputs(payload.inputs())
+                .with_outputs(payload.outputs())
+                .with_nonce(nonce)
+                .with_payload(payload.bytes())
+                .with_payload_hash_method(HashMethod::Sha512)
+                .build_pair(&*self.signer)
+                .map_err(|err| {
+                    InvalidStateError::with_message(format!(
+                        "Failed to build transaction pair: {}",
+                        err
+                    ))
+                })?,
+            None,
+        ))
     }
 }
 
@@ -85,10 +95,19 @@ impl XoBatchWorkload {
 }
 
 impl BatchWorkload for XoBatchWorkload {
-    fn next_batch(&mut self) -> Result<BatchPair, WorkloadError> {
-        Ok(BatchBuilder::new()
-            .with_transactions(vec![self.transaction_workload.next_transaction()?.take().0])
-            .build_pair(&*self.signer)?)
+    fn next_batch(
+        &mut self,
+    ) -> Result<(BatchPair, Option<ExpectedBatchResult>), InvalidStateError> {
+        let (txn, result) = self.transaction_workload.next_transaction()?;
+        Ok((
+            BatchBuilder::new()
+                .with_transactions(vec![txn.take().0])
+                .build_pair(&*self.signer)
+                .map_err(|err| {
+                    InvalidStateError::with_message(format!("Failed to build batch pair: {}", err))
+                })?,
+            result,
+        ))
     }
 }
 
@@ -102,20 +121,21 @@ struct Payload {
 }
 
 impl Payload {
-    /// Creates a Payload initialized with Action::CREATE and a randomly
+    /// Creates a Payload initialized with Action::Create and a randomly
     /// generated name.
-    pub fn new_as_create_with_random_name(rnd: &mut Hc128Rng) -> Self {
-        let length = rnd.gen_range(5, 20);
+    pub fn new_as_create_with_random_name(rnd: &mut StdRng) -> Self {
+        let length = rnd.gen_range(5..20);
 
         Payload::new_as_create(
             rnd.sample_iter(&Alphanumeric)
+                .map(char::from)
                 .take(length)
                 .collect::<String>()
                 .as_str(),
         )
     }
 
-    /// Creates a Payload initialized with Action::CREATE and the specified
+    /// Creates a Payload initialized with Action::Create and the specified
     /// name.
     pub fn new_as_create(name: &str) -> Self {
         Payload {
@@ -135,12 +155,12 @@ impl Payload {
         let mut address = Vec::new();
 
         let mut prefix_sha = Sha512::new();
-        prefix_sha.input(FAMILY_NAME);
-        address.append(&mut prefix_sha.result()[..6].to_vec());
+        prefix_sha.update(FAMILY_NAME);
+        address.append(&mut prefix_sha.finalize()[..6].to_vec());
 
         let mut name_sha = Sha512::new();
-        name_sha.input(&self.name);
-        address.append(&mut name_sha.result()[..64].to_vec());
+        name_sha.update(&self.name);
+        address.append(&mut name_sha.finalize()[..64].to_vec());
 
         address
     }
