@@ -1,5 +1,6 @@
 /*
  * Copyright 2018 Bitwise IO, Inc.
+ * Copyright 2021 Cargill Incorporated
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,31 +24,35 @@
 use std::error::Error as StdError;
 use std::fmt;
 
+#[cfg(feature = "protocol-transaction-builder")]
 use cylinder::{Signer, SigningError};
 use protobuf::Message;
+#[cfg(feature = "protocol-transaction-builder")]
 use rand::distributions::Alphanumeric;
+#[cfg(feature = "protocol-transaction-builder")]
 use rand::Rng;
+#[cfg(feature = "protocol-transaction-builder")]
 use sha2::{Digest, Sha512};
 
 use crate::protos::{
     self, FromBytes, FromNative, FromProto, IntoBytes, IntoNative, IntoProto, ProtoConversionError,
 };
 
+#[cfg(feature = "protocol-transaction-builder")]
 use super::batch::BatchBuilder;
 
+#[cfg(feature = "protocol-transaction-builder")]
 static DEFAULT_NONCE_SIZE: usize = 32;
 
 #[derive(Debug, PartialEq, Clone)]
 pub enum HashMethod {
-    // For backwards compatibility ignore clippy error
-    #[allow(clippy::upper_case_acronyms)]
-    SHA512,
+    Sha512,
 }
 
 #[derive(PartialEq, Clone)]
 pub struct TransactionHeader {
     batcher_public_key: Vec<u8>,
-    dependencies: Vec<Vec<u8>>,
+    dependencies: Vec<String>,
     family_name: String,
     family_version: String,
     inputs: Vec<Vec<u8>>,
@@ -63,7 +68,7 @@ impl TransactionHeader {
         &self.batcher_public_key
     }
 
-    pub fn dependencies(&self) -> &[Vec<u8>] {
+    pub fn dependencies(&self) -> &[String] {
         &self.dependencies
     }
 
@@ -157,11 +162,7 @@ impl FromProto<protos::transaction::TransactionHeader> for TransactionHeader {
             family_name: header.get_family_name().to_string(),
             family_version: header.get_family_version().to_string(),
             batcher_public_key: hex::decode(header.get_batcher_public_key())?,
-            dependencies: header
-                .get_dependencies()
-                .iter()
-                .map(|d| hex::decode(d).map_err(ProtoConversionError::from))
-                .collect::<Result<_, _>>()?,
+            dependencies: header.get_dependencies().to_vec(),
             inputs: header
                 .get_inputs()
                 .iter()
@@ -174,7 +175,7 @@ impl FromProto<protos::transaction::TransactionHeader> for TransactionHeader {
                 .map(|d| hex::decode(d).map_err(ProtoConversionError::from))
                 .collect::<Result<_, _>>()?,
             payload_hash: hex::decode(header.get_payload_sha512())?,
-            payload_hash_method: HashMethod::SHA512,
+            payload_hash_method: HashMethod::Sha512,
             signer_public_key: hex::decode(header.get_signer_public_key())?,
         })
     }
@@ -186,7 +187,7 @@ impl FromNative<TransactionHeader> for protos::transaction::TransactionHeader {
         proto_header.set_family_name(header.family_name().to_string());
         proto_header.set_family_version(header.family_version().to_string());
         proto_header.set_batcher_public_key(hex::encode(header.batcher_public_key()));
-        proto_header.set_dependencies(header.dependencies().iter().map(hex::encode).collect());
+        proto_header.set_dependencies(header.dependencies().into());
         proto_header.set_inputs(header.inputs().iter().map(hex::encode).collect());
         proto_header.set_nonce(String::from_utf8(header.nonce().to_vec())?);
         proto_header.set_outputs(header.outputs().iter().map(hex::encode).collect());
@@ -409,16 +410,18 @@ impl From<ProtoConversionError> for TransactionBuildError {
     }
 }
 
+#[cfg(feature = "protocol-transaction-builder")]
 impl From<SigningError> for TransactionBuildError {
     fn from(err: SigningError) -> Self {
         Self::SigningError(err.to_string())
     }
 }
 
+#[cfg(feature = "protocol-transaction-builder")]
 #[derive(Default, Clone)]
 pub struct TransactionBuilder {
     batcher_public_key: Option<Vec<u8>>,
-    dependencies: Option<Vec<Vec<u8>>>,
+    dependencies: Option<Vec<String>>,
     family_name: Option<String>,
     family_version: Option<String>,
     inputs: Option<Vec<Vec<u8>>>,
@@ -428,6 +431,7 @@ pub struct TransactionBuilder {
     payload: Option<Vec<u8>>,
 }
 
+#[cfg(feature = "protocol-transaction-builder")]
 impl TransactionBuilder {
     pub fn new() -> Self {
         TransactionBuilder::default()
@@ -438,7 +442,7 @@ impl TransactionBuilder {
         self
     }
 
-    pub fn with_dependencies(mut self, dependencies: Vec<Vec<u8>>) -> TransactionBuilder {
+    pub fn with_dependencies(mut self, dependencies: Vec<String>) -> TransactionBuilder {
         self.dependencies = Some(dependencies);
         self
     }
@@ -502,6 +506,7 @@ impl TransactionBuilder {
         let nonce = self.nonce.unwrap_or_else(|| {
             rand::thread_rng()
                 .sample_iter(&Alphanumeric)
+                .map(char::from)
                 .take(DEFAULT_NONCE_SIZE)
                 .collect::<String>()
                 .as_bytes()
@@ -517,10 +522,10 @@ impl TransactionBuilder {
         })?;
 
         let payload_hash = match payload_hash_method {
-            HashMethod::SHA512 => {
+            HashMethod::Sha512 => {
                 let mut hasher = Sha512::new();
-                hasher.input(&payload);
-                hasher.result().to_vec()
+                hasher.update(&payload);
+                hasher.finalize().to_vec()
             }
         };
 
@@ -610,16 +615,16 @@ mod tests {
             .expect("Failed to get signer public key");
 
         let payload_hash = match pair.header().payload_hash_method() {
-            HashMethod::SHA512 => {
+            HashMethod::Sha512 => {
                 let mut hasher = Sha512::new();
-                hasher.input(&pair.transaction().payload());
-                hasher.result().to_vec()
+                hasher.update(&pair.transaction().payload());
+                hasher.finalize().to_vec()
             }
         };
 
         assert_eq!(KEY1, hex::encode(pair.header().batcher_public_key()));
         assert_eq!(
-            vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap(),],
+            vec![KEY2.to_string(), KEY3.to_string()],
             pair.header().dependencies()
         );
         assert_eq!(FAMILY_NAME, pair.header.family_name());
@@ -639,17 +644,18 @@ mod tests {
             pair.header().outputs()
         );
         assert_eq!(payload_hash, pair.header().payload_hash());
-        assert_eq!(HashMethod::SHA512, *pair.header().payload_hash_method());
+        assert_eq!(HashMethod::Sha512, *pair.header().payload_hash_method());
         assert_eq!(signer_pub_key.as_slice(), pair.header().signer_public_key());
     }
 
+    #[cfg(feature = "protocol-transaction-builder")]
     #[test]
     fn transaction_builder_chain() {
         let signer = new_signer();
 
         let pair = TransactionBuilder::new()
             .with_batcher_public_key(hex::decode(KEY1).unwrap())
-            .with_dependencies(vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap()])
+            .with_dependencies(vec![KEY2.to_string(), KEY3.to_string()])
             .with_family_name(FAMILY_NAME.to_string())
             .with_family_version(FAMILY_VERSION.to_string())
             .with_inputs(vec![
@@ -661,7 +667,7 @@ mod tests {
                 hex::decode(KEY6).unwrap(),
                 hex::decode(&KEY7[0..4]).unwrap(),
             ])
-            .with_payload_hash_method(HashMethod::SHA512)
+            .with_payload_hash_method(HashMethod::Sha512)
             .with_payload(BYTES2.to_vec())
             .build_pair(&*signer)
             .unwrap();
@@ -669,14 +675,14 @@ mod tests {
         check_builder_transaction(&*signer, &pair);
     }
 
+    #[cfg(feature = "protocol-transaction-builder")]
     #[test]
     fn transaction_builder_seperate() {
         let signer = new_signer();
 
         let mut builder = TransactionBuilder::new();
         builder = builder.with_batcher_public_key(hex::decode(KEY1).unwrap());
-        builder =
-            builder.with_dependencies(vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap()]);
+        builder = builder.with_dependencies(vec![KEY2.to_string(), KEY3.to_string()]);
         builder = builder.with_family_name(FAMILY_NAME.to_string());
         builder = builder.with_family_version(FAMILY_VERSION.to_string());
         builder = builder.with_inputs(vec![
@@ -688,7 +694,7 @@ mod tests {
             hex::decode(KEY6).unwrap(),
             hex::decode(&KEY7[0..4]).unwrap(),
         ]);
-        builder = builder.with_payload_hash_method(HashMethod::SHA512);
+        builder = builder.with_payload_hash_method(HashMethod::Sha512);
         builder = builder.with_payload(BYTES2.to_vec());
         let pair = builder.build_pair(&*signer).unwrap();
 
@@ -699,7 +705,7 @@ mod tests {
     fn transaction_header_fields() {
         let header = TransactionHeader {
             batcher_public_key: hex::decode(KEY1).unwrap(),
-            dependencies: vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap()],
+            dependencies: vec![KEY2.to_string(), KEY3.to_string()],
             family_name: FAMILY_NAME.to_string(),
             family_version: FAMILY_VERSION.to_string(),
             inputs: vec![
@@ -712,12 +718,12 @@ mod tests {
                 hex::decode(&KEY7[0..4]).unwrap(),
             ],
             payload_hash: hex::decode(HASH).unwrap(),
-            payload_hash_method: HashMethod::SHA512,
+            payload_hash_method: HashMethod::Sha512,
             signer_public_key: hex::decode(KEY8).unwrap(),
         };
         assert_eq!(KEY1, hex::encode(header.batcher_public_key()));
         assert_eq!(
-            vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap(),],
+            vec![KEY2.to_string(), KEY3.to_string()],
             header.dependencies()
         );
         assert_eq!(FAMILY_NAME, header.family_name());
@@ -737,7 +743,7 @@ mod tests {
             header.outputs()
         );
         assert_eq!(HASH, hex::encode(header.payload_hash()));
-        assert_eq!(HashMethod::SHA512, *header.payload_hash_method());
+        assert_eq!(HashMethod::Sha512, *header.payload_hash_method());
         assert_eq!(KEY8, hex::encode(header.signer_public_key()));
     }
 
@@ -746,7 +752,7 @@ mod tests {
     fn transaction_header_bytes() {
         let original = TransactionHeader {
             batcher_public_key: hex::decode(KEY1).unwrap(),
-            dependencies: vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap()],
+            dependencies: vec![KEY2.to_string(), KEY3.to_string()],
             family_name: FAMILY_NAME.to_string(),
             family_version: FAMILY_VERSION.to_string(),
             inputs: vec![
@@ -759,7 +765,7 @@ mod tests {
                 hex::decode(&KEY7[0..4]).unwrap(),
             ],
             payload_hash: hex::decode(HASH).unwrap(),
-            payload_hash_method: HashMethod::SHA512,
+            payload_hash_method: HashMethod::Sha512,
             signer_public_key: hex::decode(KEY8).unwrap(),
         };
 
@@ -823,7 +829,7 @@ mod tests {
 
         assert_eq!(KEY1, hex::encode(header.batcher_public_key()));
         assert_eq!(
-            vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap()],
+            vec![KEY2.to_string(), KEY3.to_string()],
             header.dependencies()
         );
         assert_eq!(FAMILY_NAME, header.family_name());
@@ -844,7 +850,7 @@ mod tests {
             header.outputs()
         );
         assert_eq!(hex::decode(HASH).unwrap(), header.payload_hash());
-        assert_eq!(HashMethod::SHA512, *header.payload_hash_method());
+        assert_eq!(HashMethod::Sha512, *header.payload_hash_method());
         assert_eq!(hex::decode(KEY8).unwrap(), header.signer_public_key());
     }
 
@@ -917,12 +923,13 @@ mod benchmarks {
     static SIGNATURE1: &str =
         "sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1sig1";
 
+    #[cfg(feature = "protocol-transaction-builder")]
     #[bench]
     fn bench_transaction_builder(b: &mut Bencher) {
         let signer = new_signer();
         let transaction = TransactionBuilder::new()
             .with_batcher_public_key(hex::decode(KEY1).unwrap())
-            .with_dependencies(vec![hex::decode(KEY2).unwrap(), hex::decode(KEY3).unwrap()])
+            .with_dependencies(vec![KEY2.to_string(), KEY3.to_string()])
             .with_family_name(FAMILY_NAME.to_string())
             .with_family_version(FAMILY_VERSION.to_string())
             .with_inputs(vec![
@@ -934,7 +941,7 @@ mod benchmarks {
                 hex::decode(KEY6).unwrap(),
                 hex::decode(&KEY7[0..4]).unwrap(),
             ])
-            .with_payload_hash_method(HashMethod::SHA512)
+            .with_payload_hash_method(HashMethod::Sha512)
             .with_payload(BYTES2.to_vec());
 
         b.iter(|| transaction.clone().build_pair(&*signer));
@@ -953,7 +960,7 @@ mod benchmarks {
     fn bench_txn_header_into_proto(b: &mut Bencher) {
         let header = TransactionHeader {
             batcher_public_key: hex::decode(KEY1).unwrap(),
-            dependencies: vec![hex::decode(KEY2).unwrap()],
+            dependencies: vec![KEY2.to_string()],
             family_name: FAMILY_NAME.to_string(),
             family_version: FAMILY_VERSION.to_string(),
             inputs: vec![
@@ -966,7 +973,7 @@ mod benchmarks {
                 hex::decode(&KEY7[0..4]).unwrap(),
             ],
             payload_hash: hex::decode(HASH).unwrap(),
-            payload_hash_method: HashMethod::SHA512,
+            payload_hash_method: HashMethod::Sha512,
             signer_public_key: hex::decode(KEY8).unwrap(),
         };
 
