@@ -203,7 +203,7 @@ fn as_sawtooth_header(header: &TransactionHeader) -> SawtoothTxnHeader {
     sawtooth_header.set_family_version(header.family_version().to_owned());
     sawtooth_header.set_signer_public_key(hex::encode(&header.signer_public_key()));
     sawtooth_header.set_batcher_public_key(hex::encode(&header.batcher_public_key()));
-    sawtooth_header.set_dependencies(header.dependencies().iter().map(hex::encode).collect());
+    sawtooth_header.set_dependencies(header.dependencies().into());
     sawtooth_header.set_inputs(header.inputs().iter().map(hex::encode).collect());
     sawtooth_header.set_outputs(header.outputs().iter().map(hex::encode).collect());
     sawtooth_header.set_nonce(hex::encode(&header.nonce()));
@@ -411,7 +411,7 @@ mod xo_compat_test {
             .with_inputs(vec![hex::decode(&game_address).unwrap()])
             .with_nonce("test_nonce".as_bytes().to_vec())
             .with_outputs(vec![hex::decode(&game_address).unwrap()])
-            .with_payload_hash_method(HashMethod::SHA512)
+            .with_payload_hash_method(HashMethod::Sha512)
             .with_payload(payload.as_bytes().to_vec())
             .build_pair(signer)
             .expect("The TransactionBuilder was not given the correct items");
@@ -448,8 +448,8 @@ mod xo_compat_test {
 
     fn calculate_game_address<S: AsRef<[u8]>>(name: S) -> String {
         let mut sha = Sha512::default();
-        sha.input(name);
-        "5b7349".to_owned() + &hex::encode(&sha.result())[..64]
+        sha.update(name);
+        "5b7349".to_owned() + &hex::encode(&sha.finalize())[..64]
     }
 
     fn initial_db_root(db: &dyn Database) -> String {
